/*!
 *  ======== Archive.xdc ========
 *  A target filter that fixes long archiver command lines
 *
 *  This filter can be used with TI toolchains to replace
 *  very long commands lines that result from trying to create an
 *  archive with hundreds of files.
 *
 *  This filter works by replacing the command line normally emitted by
 *  the target with one that references a generated file containing the
 *  list of file to archive.
 *
 *  @a(USAGE)
 *  To use this filter, you must configure the profiles of the TI targets
 *  that are used in your `config.bld` script.  Suppose, for example, that
 *  you need this filter for the `ti.targets.C64P` target.  By adding the
 *  following XDCscript to the end of `config.bld`, this filter will "fix"
 *  all uses of the TI archiver to use its `-@` option and allow an
 *  arbitrary number of files to be added to any TI library.
 *  @p(code)
 *      var archiveFilter = {
 *          moduleName: "basic.filter.Archive",
 *      };
 *
 *      var C64P = xdc.module("ti.targets.C64P");
 *      for each (var profile in C64P.profiles) {
 *          profile.filters[profile.filters.length++] = archiveFilter;
 *      }
 *  @p
 */
metaonly module Archive inherits xdc.bld.ITargetFilter
{
}
