var Me;
var Pkg = xdc.module('xdc.bld.PackageContents');

var RE = new RegExp(
    "[^ ]+/package_" + Pkg.name + ".o[^ ]*".replace(/\./g, "\."));

/*
 *  ======== module$meta$init ========
 */
function module$meta$init()
{
    Me = this;
}

/*
 *  ======== instance$meta$init ========
 */
function instance$meta$init()
{
    var file = new java.io.File(this.makefileName);
    file["delete"]();

    this.$private.out = new java.io.BufferedWriter(
        new java.io.FileWriter(file));

    this.$private.targets = {};
    this.$private.dirs = {};

    /* Initialize the header generation. */ 
    this.$private.headerGenerated = false;
}

/*
 *  ======== archive ========
 */
function archive(container, lib, objList, archArgs, res)
{
    /* remove generated XDC build object file */
    res.cmds = res.cmds.replace(RE, "");
    objList = objList.replace(RE, "");

    var out = this.$private.out;

    if (this.$private.libDef != true) {
	this.$private.libDef = true;
	
	out.write("#\n# Set LIBDIR to the directory where libraries are created\n#\n");
	out.write("LIBDIR ?= "
		  + container.substr(0, container.lastIndexOf('/')) + "\n\n");
    }

    var goal = "$(LIBDIR)" + container.substring(container.lastIndexOf('/'));

    out.write(".libraries: " + goal + "\n");
    _genRules(this, lib.target, goal, objList, res.cmds);
}

/*
 *  ======== compile ========
 */
function compile(container, target, goalFile, compArgs, res)
{
    /* remove generated XDC build object file */
    if (goalFile.match(RE)) {
        return;
    }

    _genRules(this, target, goalFile,
        compArgs.srcPrefix + compArgs.base + compArgs.srcSuffix, res.cmds);
}

/*
 *  ======== link ========
 */
function link(container, cfgBase, prog, objList, linkArgs, res)
{
    /* ensure executables are built _after_ libraries */
    this.$private.out.write(container + ": .libraries\n");
    
    _genRules(this, prog.target, container, objList, res.cmds);
}

/*
 *  ======== getDefs ========
 */
function getDefs()
{
    /* generate directories here because we don't have a better hook to know
     * when filter generation of objects is done.
     */
    _genDirs(this.$private.dirs, this.$private.out);

    return (null);
}

/*
 *  ======== _genRules ========
 */
function _genRules(filter, target, goal, prereq, cmds)
{
    var out = filter.$private.out;
    
    /* generate top of make file (just once) */
    if (filter.$private.headerGenerated != true) {
        filter.$private.headerGenerated = true;
        _genHeader(out);
    }

    /* add any directory prefix to the list of directories to create */
    _addDir(goal, filter.$private.dirs);
    
    /* if we haven't already, generate default definition of rootDir */
    if (filter.$private.targets[target.$name] == null) {
	filter.$private.targets[target.$name] = 1;
        out.write("#\n# default " + target.name
            + " compiler installation location\n#\n");
	out.write(target.$name + ".rootDir ?= " + target.rootDir + "\n\n");
    }
    
    /* add goal to all and a clean rule to remove this goal */
    out.write("#\n# ======== " + goal + " ========\n#\n");
    out.write("all: " + goal + "\n");
    out.write("clean::\n\t-$(RM) " + goal + "\n");

    /* generate rule for goal with prerequisites */
    out.write("\n" + goal + ": " + prereq + "\n");
    cmds = cmds.replace(/\$\(rootDir\)/g, "$(" + target.$name + ".rootDir)");
    var rules = cmds.split('\n');
    for (var i = 0; i < rules.length; i++) {
        var rule = rules[i];
        if (rule.indexOf("$(MKDEP)") != -1
            || rule.indexOf("$(FIXDEP)") != -1) {
            continue;
        }
        out.write("\t" + rules[i] + "\n");
    }
    out.write("\n");

    out.flush();
}

/*
 *  ======== _genHeader ========
 */
function _genHeader(out)
{
    var xdcpath = String(environment["xdc.path"]).replace(/;+$/, "");

    out.write("# This file was generated by the " + Me + " filter on\n# "
        + Date());
    out.write("\n\n#\n# Set XDCPATH to your package's repository\n#\n");
    out.write("XDCPATH ?= " + xdcpath + "\n\n");
    out.write("#\n# Macro definitions referenced by the rules below\n#\n");
    out.write("empty =\n");
    out.write("space =$(empty) $(empty)\n");
    out.write("XDCCFGDIR = " + Pkg.cfgDir + "\n");
    out.write("XDCINCS = -I. -I$(strip $(subst ;, -I,$(subst $(space),\\$(space),$(XDCPATH))))\n");
    out.write("MKDIR = mkdir\n");
    out.write("RM = rm -f\n\n");
    out.write(".PHONY: all clean .libraries\n\n");
    
    out.write("all:;\n\n");
    out.flush();
}

/*
 *  ======== _addDir ========
 *  For the file specified, add its directory prefix to the dirs hash
 */
function _addDir(file, dirs)
{
    if (file.lastIndexOf('/') != -1 || file.lastIndexOf('\\') != -1) {
        var dir = file.replace(/[\/\\][^\/\\]*$/, "");
        if (dir != "") {
            dirs[dir] = dir;
        }
    }
}

/*
 *  ======== _genDirs ========
 *  Generate rules to automatically create all directories necessary
 *  for the build *and* the rules to remove the directories during a
 *  clean.
 */
function _genDirs(dirs, out)
{
    out.write("\n#\n# create all necessary output sub-directories\n\#\n");
    /*  Create an array of all directories (including sub-directories) that
     *  need to be created.
     */
    var dirSet = {};
    var dirArr = [];
    for (var dir in dirs) {
        var da = dir.split(/[\\\/]+/);
        var tmp = "";
        for (var i = 0; i < da.length; i++) {
            if (da[i].length > 0 && da[i] != ".") {
                tmp = tmp.concat(da[i] + "/");
                if (dirSet[tmp] != 1) {
                    dirSet[tmp] = 1;
                    dirArr[dirArr.length] = tmp.substring(0, tmp.length - 1);
                }
            }
        }
    }

    /*  Generate rules to create the directories determined above.
     *
     *  Note that the directories are not created when the make command is
     *  "clean".  This is not just an optimization:  GNU make seems to keep
     *  directories open when they appear in a "wildcard" expression.
     *  Since Win32 does not allow deletion of a file/directory while it
     *  is open, it is impossible to remove the directories with a clean
     *  command unless we don't reference the directories during a clean.
     */
    out.write("ifneq (clean,$(MAKECMDGOALS))\n");
    for (var i = 0; i < dirArr.length; i++) {
        var dir = dirArr[i];
        /* skip over directories like "../" */
        if (dir.match(/^[\.\\\/]*$/) == null) {
            out.write("ifeq (,$(wildcard " + dir + "))\n");
            out.write("    $(shell $(MKDIR) " + dir + ")\n");
            out.write("endif\n");
        }
    }

    out.write("endif\n\n");
    out.flush();
}
