/*
 *  ======== Time.xs ========
 */

var timeCmd = "";   /* global time command suffix */

/*
 *  ======== module$meta$init ========
 *  This function is called when this module is first loaded
 */
function module$meta$init()
{
    var Time = this;    /* 'this' is the Time module object itself */

    /* initialize the global time command suffix */
    var outfile = Time.outputFile ? (">> " + Time.outputFile) : "";
    timeCmd = "`$(XDCROOT)/bin/date '+%R:%S.%N'`" + outfile + "\n";
    if (Build.hostOS != Build.WINDOWS) {
        timeCmd = "`date +%R:%S.%N`" + outfile + "\n";
    }
}

/*
 *  ======== compile ========
 */
function compile(container, target, goalFile, compArgs, res)
{
    var ccPat = this.$private.ccPat;
    if (ccPat == null) {
        _init(this, target);
        ccPat = this.$private.ccPat;
    }

    res.cmds = _insertTime(res.cmds, ccPat, "compile begin: ",
        "compile end  : ");
}

/*
 *  ======== archive ========
 */
function archive(container, lib, objList, archArgs, res)
{
    res.cmds = _insertTime(res.cmds, this.$private.arPat, "archive begin: ",
            "archive end  : ");
}

/*
 *  ======== link ========
 */
function link(container, cfgBase, prog, objList, linkArgs, res)
{
    res.cmds = _insertTime(res.cmds, this.$private.lnkPat, "link begin: ",
            "link end  : ");
}

/*
 *  ======== _insertTime ========
 *  Wrap just the origCmd within cmdString with appropriate timing commands
 */
function _insertTime(cmdString, cmdPattern, beginString, endString)
{
    if (cmdPattern == null) {
        return (cmdString);
    }

    var beginTime = '$(ECHO) "' + beginString + '"' + timeCmd;
    var endTime = '$(ECHO) "' + endString + '"' + timeCmd;
    
    /* wrap just cmdPattern with begin/end time logging commands  */
    var resCmd = "";
    var cmds = cmdString.split('\n');
    for (var i = 0; i < cmds.length; i++) {
        if (cmds[i].match(cmdPattern)) {
//            print("matched cmd: '" + cmds[i].split(/\s/)[0] + "'");
            resCmd = resCmd + beginTime + cmds[i] + '\n' + endTime;
        }
        else {
            resCmd = resCmd + cmds[i] + '\n';
	}
    }

    return (resCmd);
}

/*
 *  ======== _init ========
 *  initialize target-specific patterns for the compile, link, and archive
 *  commands
 */
function _init(filter, target)
{
    /* we cheat here since most targets prefix toolchain commands with
     * $(rootDir)/
     */
    filter.$private.ccPat = "^\\$\\(rootDir\\)";
    filter.$private.arPat = "^\\$\\(rootDir\\)";
    filter.$private.lnkPat = "^\\$\\(rootDir\\)";
}
